@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,8 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _uim_agent_helper_message_h_included_
-#define _uim_agent_helper_message_h_included_
+#ifndef UIM_AGENT_HELPER_MESSAGE_H
+#define UIM_AGENT_HELPER_MESSAGE_H
 
 #include <stdio.h>
 #include <stdlib.h>
